using System;
using System.Collections.Generic;
using System.Collections;
using System.Text;
using System.Text.RegularExpressions;

namespace KyotoUnivEngHywr
{
	/// <summary>
	/// KyotoHydro で作成した小道具関数。
	/// </summary>
	public class KhUtility
	{
		static Regex commentString = new Regex(@"#.*$");
		static Regex whiteSpaces = new Regex(@"\s+");
		static Regex whiteLine = new Regex(@"^\s*$");
		static Regex commentLine = new Regex(@"^\s*#.*$");
		static Regex headWhiteSpaces = new Regex(@"^\s+");
		static Regex tailWhiteSpaces = new Regex(@"\s+$");

		private KhUtility()
		{
		}

		/// <summary>
		/// 行頭から0個以上の空白文字があった後に # がある行は読み
		/// 飛ばし、行の途中に # があるとそこから行末までがコメント
		/// と見なして削除するというルールで、ストリーム sr から一
		/// 行読み取ります。
		/// </summary>
		/// <param name="sr">入力。読み取りデータが入っているストリー
		/// ム</param>
		/// <param name="RequireData">入力。
		/// 行頭から0個以上の空白文字があった後に # がある行は読み
		/// 飛ばすというルールで読み進めて行った結果、ストリームの
		/// 終端に達してしまった場合に、どのような動作をするかを指
		/// 定します。
		///
		/// <para>
		/// この引数 RequiredData が true なら、
		/// ストリームの最後に達してしまったら、
		/// 例外を送出します。</para>
		///
		/// <para>
		/// この引数が RequiredData が false なら、
		/// ストリームの最後に達してしまったら、nullを返します。
		/// </para>
		/// </param>
		///
		/// <returns>読み取った文字列</returns>
		/// <example>
		/// 読み込むデータがあるはずのときは、つぎのように使用しま
		/// す。
		/// <code>
		/// string line;
		/// line = KhUtility.ReadLine(sr, true);
		/// </code>
		/// 読み込むデータが無くなるまで作業をするときは、
		/// つぎのように使用します。
		/// <code>
		/// string line;
		/// while ((line = KhUtility.ReadLine(sr, false)) != null)
		/// {
		///	 ... some work for line;
		/// }
		/// </code>
		/// </example>
		public static string ReadLine(
		  System.IO.StreamReader sr,
		  bool RequireData)
		{
			string line;
			while (true)
			{
				line = sr.ReadLine();
				if (line == null)
				{
					if (RequireData == true)
					{
						throw new Exception(
						  "End of file was encountered.");
					}
					else
					{
						return null;
					}
				}
				else if (line.Length == 0)
				{
					break;
				}
				else if (Regex.IsMatch(line, @"^\s*#.*$"))
				{
					continue;
				}
				else if (line[0] != '#')
				{
					line = commentString.Replace(line, "");
					break;
				}
			}
			return line;
		}

		/// <summary>
		/// 行頭から0個以上の空白文字があった後に # がある行は読み
		/// 飛ばし、行の途中に # があるとそこから行末までがコメント
		/// と見なして削除するというルールで、ストリーム sr から一
		/// 行読み取ります。
		/// 途中でファイルの終端に到達した場合は、null を返します。
		/// </summary>
		/// <param name="sr">入力。読み取りデータが入っているストリー
		/// ム</param>
		///
		/// <returns>読み取った文字列</returns>
		/// <example>
		/// 読み込むデータが無くなるまで作業をするときは、
		/// つぎのように使用します。
		/// <code>
		/// string line;
		/// while ((line = KhUtility.ReadLine(sr)) != null)
		/// {
		///	 ... some work for line;
		/// }
		/// </code>
		/// </example>
		public static string ReadLine(System.IO.StreamReader sr)
		{
			return ReadLine(sr, false);
		}

		/// <summary>
		/// 与えられた文字列の先頭や末尾の空白文字列を
		/// 削除して返します。
		/// 
		/// <para>たとえば、" abc de " を引数として与えると, 
		/// "abc de" を返します。</para>
		///
		/// </summary>
		/// <param name="line">入力。処理対象の文字列。
		/// </param>
		/// <returns>行頭や行末の空白文字列を取り除いた文字列
		/// </returns>
		public static string DeleteWhites(string line)
		{
			string nline;
			nline = headWhiteSpaces.Replace(line, "");
			nline = tailWhiteSpaces.Replace(nline, "");
			return nline;
		}

		/// <summary>
		/// 行頭から0個以上の空白文字があってその後に # がある行は
		/// 読み飛ばし、行の途中に # があるとそこから行末までがコメ
		/// ントと見なして削除するというルールで、ストリーム sr か
		/// ら一行読み取り、行頭や行末の空白文字列を削除します。
		/// こうして得られた空白文字列でない文字列を、空白文字列を
		/// 区切り文字列と見なして単語に切り分けた結果を返します。
		/// 途中でファイルの終端に到達した場合は、例外を送出します。
		/// </summary>
		/// <param name="sr">入力。読み取りデータが入っているスト
		/// リーム</param>
		/// <returns>切り分けて得られた単語の配列。配列のサイズは、
		/// 1以上であることが保証されています。</returns>
		public static string[] GetWords(System.IO.StreamReader sr)
		{
			string line;
			while (true)
			{
				line = sr.ReadLine();
				if (line == null)
				{
					throw new Exception(
							  "End of file was encountered.");
				}
				else if (line.Length == 0 ||
				  Regex.IsMatch(line, @"^\s*#.*$"))
				{
					continue;
				}
				line = DeleteWhites(commentString.Replace(line, ""));
				break;
			}
			string[] words = Split(line);
			return words;
		}

		/// <summary>
		/// 与えられた文字列の先頭や末尾の空白文字列を削除した後で、
		/// 空白文字列を区切り文字列とみなして単語に切り分けた結果を
		/// 返します。
		///
		/// <para>
		/// たとえば、" 11 2  3 " を引数として与えると、
		/// "11", "2", "3"
		/// の3個の文字列を要素とする配列を返します。
		/// </para>
		///
		/// <para>
		/// 引数に、空白文字列を与えると、null を返します。
		/// </para>
		/// </summary>
		/// <param name="line">入力。処理対象の文字列</param>
		/// <returns>切り分けた単語の配列または null</returns>
		public static string[] Split(string line)
		{
			string[] words;
			string nline = DeleteWhites(line);
			if (nline.Length == 0)
			{
				return null;
			}
			else
			{
				words = whiteSpaces.Split(nline);
			}
			return words;
		}

		/// <summary>
		/// 与えられた文字列が空白文字列か検査します。
		/// </summary>
		/// <param name="line">入力。検査対象の文字列</param>
		/// <returns>空白文字列なら true、
		/// そうでなければ、false</returns>
		public static bool IsWhiteLine(string line)
		{
			return Regex.IsMatch(line, @"^\s*$");
		}

		/// <summary>
		/// 行列 A の内容を標準出力に印字します。
		/// </summary>
		/// <param name="m">入力。行列 A の行数</param>
		/// <param name="n">入力。行列 A の列数</param>
		/// <param name="a">入力。行列 A を格納している配列</param>
		/// <param name="format">行列 A の要素を印字するのに
		///使用するフォーマット文字列。例. " {0, 9:F4}"
		/// </param>
		public static void PrintMatrix(
		  int m, int n, double[,] a, string format
		)
		{
			for (int i = 0; i < m; i++)
			{
				Console.Write("   ");
				for (int j = 0; j < n; j++)
				{
					Console.Write(format, a[i, j]);
				}
				Console.WriteLine("");
			}
		}

		/// <summary>
		/// 行列 A の内容を標準出力に印字します。
		/// 列数が大きいときに折り返して印字します。
		/// </summary>
		/// <param name="m">入力。行列 A の行数</param>
		/// <param name="n">入力。行列 A の列数</param>
		/// <param name="k">入力。一行に最大何個の要素を書くか、
		/// その要素数。
		/// n > k であると想定していますが、
		/// そうでなくても構いません。</param>
		/// <param name="c">入力。各行の最初の k 個の成分を出力する
		/// ときに出力する行頭文字、例えば ':'</param>
		/// <param name="a">入力。行列 A を格納している配列</param>
		/// <param name="format">入力。行列 A の要素を印字するのに
		/// 使用するフォーマット文字列。例. " {0, 9:F4}"
		/// </param>
		public static void PrintMatrix(
		  int m, int n, int k, char c, double[,] a, string format
		)
		{
			for (int i = 0; i < m; i++)
			{
				int js = 0;
				Console.Write(" " + c + " ");
				while (js < n)
				{
					int je = Math.Min(n, js + k);
					for (int j = js; j < je; j++)
					{
						Console.Write(format, a[i, j]);
					}
					Console.WriteLine("");
					if (je >= n)
					{
						break;
					}
					else
					{
						js = je;
						Console.Write("   ");
					}
				}
			}
		}


		/// <summary>
		/// 行列 A の内容を標準出力に印字します。
		/// 要素を印字するとき、
		/// 標準のフォーマット " {0, 9:F4}" を使用します。
		/// </summary>
		/// <param name="m">入力。行列 A の行数</param>
		/// <param name="n">入力。行列 A の列数</param>
		/// <param name="a">入力。行列 A を格納している配列</param>
		public static void PrintMatrix(
		  int m, int n, double[,] a
		)
		{
			for (int i = 0; i < m; i++)
			{
				Console.Write("   ");
				for (int j = 0; j < n; j++)
				{
					Console.Write(" {0, 9:F4}", a[i, j]);
				}
				Console.WriteLine("");
			}
		}

		/// <summary>
		/// 行列 A の内容を標準出力に印字します。
		/// 要素を印字するとき、
		/// 標準のフォーマット " {0, 9:F4}" を使用します。
		/// 列数が大きいとき、折り返して印字します。
		/// </summary>
		/// <param name="m">入力。行列 A の行数</param>
		/// <param name="n">入力。行列 A の列数</param>
		/// <param name="k">入力。一行に最大何個の要素を書くか、
		///その要素数。
		///n > k であると想定していますが、そうでなくても
		///構いません。</param>
		/// <param name="c">入力。各行の最初の k 個の成分を出力する
		///ときに出力する行頭文字、例えば ':'</param>
		/// <param name="a">入力。行列 A を格納している配列</param>
		public static void PrintMatrix(
		  int m, int n, int k, char c, double[,] a
		)
		{
			PrintMatrix(m, n, k, c, a, " {0, 9:F4}");
		}

		/// <summary>
		/// ベクトル V の内容を印字します。
		/// </summary>
		/// <param name="n">入力。ベクトル V の次元</param>
		/// <param name="v">入力。ベクトル V を格納している配列
		/// </param>
		/// <param name="format">入力。ベクトル V の要素を
		///印刷するために使用する書式文字列。
		///例. " {0, 9:F4}"</param>
		public static void PrintVec(
			int n, double[] v, string format
		)
		{
			Console.Write("   ");
			for (int i = 0; i < n; i++)
				Console.Write(format, v[i]);

			Console.WriteLine("");
		}

		/// <summary>
		/// ベクトル V の内容を標準出力に印字します。
		/// 要素を印字するとき、
		/// 標準のフォーマット " {0, 9:F4}" を使用します。
		/// </summary>
		/// <param name="n">入力。ベクトル V の次元</param>
		/// <param name="v">入力。ベクトル V を格納している配列
		/// </param>
		public static void PrintVec(
			int n, double[] v
		)
		{
			PrintVec(n, v, " {0, 9:F4}");
		}

		/// <summary>
		/// ベクトル V の内容を標準出力に印字します。
		/// 成分の個数が大きいと折り返して印字します。
		/// </summary>
		/// <param name="n">入力。ベクトル V の次元</param>
		/// <param name="k">入力。n が大きいときに一行に何個の要素
		/// を書くか</param>
		/// <param name="c">入力。各行の最初の k 個の成分を出力する
		///ときに出力する行頭文字、例えば ':'</param>
		/// <param name="v">入力。ベクトル V を格納している配列</param>
		/// <param name="format">入力。ベクトル V の要素を
		///印刷するために使用する書式文字列。
		///例. " {0, 9:F4}"</param>
		public static void PrintVec(
			int n, int k, char c, double[] v, string format
		)
		{
			Console.Write(" " + c + " ");
			int js = 0;
			while (js < n)
			{
				int je = Math.Min(n, js + k);
				for (int j = js; j < je; j++)
				{
					Console.Write(format, v[j]);
				}
				Console.WriteLine("");
				if (je >= n)
				{
					break;
				}
				else
				{
					js = je;
					Console.Write("   ");
				}
			}
		}

		/// <summary>
		/// ベクトル V の内容を標準出力に印字します。要素を印字するとき、
		/// 標準のフォーマット " {0, 9:F4}" を使用します。
		/// 成分の個数が大きいと折り返して印字します。
		/// </summary>
		/// <param name="n">入力。ベクトル V の次元</param>
		/// <param name="k">入力。n が大きいときに一行に何個の要素を
		/// 書くか</param>
		/// <param name="c">入力。各行の最初の k 
		/// 個の成分を出力するときに出力する行頭文字、
		/// 例えば ':'</param>
		/// <param name="v">入力。ベクトル V を格納している配列</param>
		public static void PrintVec(
			int n, int k, char c, double[] v
		)
		{
			PrintVec(n, k, c, v, " {0, 9:F4}");
		}

		/// <summary>
		/// 行列 A の内容をストリームに印字します。
		/// </summary>
		/// <param name="m">入力。行列 A の行数</param>
		/// <param name="n">入力。行列 A の列数</param>
		/// <param name="a">入力。行列 A を格納している配列</param>
		/// <param name="format">入力。行列 A の要素を印字するのに
		/// 使用するフォーマット文字列。例. " {0, 9:F4}"</param>
		/// <param name="sw">入力。印字先のストリーム</param>
		public static void PrintMatrix(
		  int m, int n, double[,] a, string format,
		   System.IO.StreamWriter sw
		)
		{
			for (int i = 0; i < m; i++)
			{
				sw.Write("   ");
				for (int j = 0; j < n; j++)
				{
					sw.Write(format, a[i, j]);
				}
				sw.WriteLine("");
			}
		}

		/// <summary>
		/// 行列 A の内容をストリームに印字する。
		/// 列数が大きいときに折り返して印字します。
		/// </summary>
		/// <param name="m">入力。行列 A の行数</param>
		/// <param name="n">入力。行列 A の列数</param>
		/// <param name="k">入力。一行に最大何個の要素を書くか、その要素数。
		///n > k であると想定していますが、そうでなくても
		///構いません。</param>
		/// <param name="c">入力。各行の最初の k 個の成分を出力する
		///ときに出力する行頭文字、例えば ':'</param>
		/// <param name="a">入力。行列 A を格納している配列</param>
		/// <param name="format">入力。行列 A の要素を印字するのに
		///使用するフォーマット文字列。例. " {0, 9:F4}"</param>
		/// <param name="sw">入力。印字先のストリーム</param>
		public static void PrintMatrix(
		  int m, int n, int k, char c, double[,] a, string format,
		  System.IO.StreamWriter sw
		)
		{
			for (int i = 0; i < m; i++)
			{
				int js = 0;
				sw.Write(" " + c + " ");
				while (js < n)
				{
					int je = Math.Min(n, js + k);
					for (int j = js; j < je; j++)
					{
						sw.Write(format, a[i, j]);
					}
					sw.WriteLine("");
					if (je >= n)
					{
						break;
					}
					else
					{
						js = je;
						sw.Write("   ");
					}
				}
			}
		}


		/// <summary>
		/// 行列 A の内容をストリームに印字します。
		///要素を印字するとき、
		///標準のフォーマット " {0, 9:F4}" を使用します。
		/// </summary>
		/// <param name="m">入力。行列 A の行数</param>
		/// <param name="n">入力。行列 A の列数</param>
		/// <param name="a">入力。行列 A を格納している配列</param>
		/// <param name="sw">入力。印字先のストリーム</param>
		public static void PrintMatrix(
		  int m, int n, double[,] a,
		  System.IO.StreamWriter sw
		)
		{
			for (int i = 0; i < m; i++)
			{
				sw.Write("   ");
				for (int j = 0; j < n; j++)
				{
					sw.Write(" {0, 9:F4}", a[i, j]);
				}
				sw.WriteLine("");
			}
		}

		/// <summary>
		/// 行列 A の内容をストリームに印字します。
		///要素を印字するとき、
		///標準のフォーマット " {0, 9:F4}" を使用します。
		///列数が大きいとき、折り返して印字します。
		/// </summary>
		/// <param name="m">入力。行列 A の行数</param>
		/// <param name="n">入力。行列 A の列数</param>
		/// <param name="k">入力。一行に最大何個の要素を書くか、
		///その要素数。
		///n > k であると想定していますが、そうでなくても
		///構いません。</param>
		/// <param name="c">入力。各行の最初の k 個の成分を出力する
		///ときに出力する行頭文字、例えば ':'</param>
		/// <param name="a">入力。行列 A を格納している配列</param>
		/// <param name="sw">入力。印字先のストリーム</param>
		public static void PrintMatrix(
		  int m, int n, int k, char c, double[,] a,
		  System.IO.StreamWriter sw
		)
		{
			PrintMatrix(m, n, k, c, a, " {0, 9:F4}", sw);
		}

		/// <summary>
		/// ベクトル V の内容をストリームに印字します。
		/// </summary>
		/// <param name="n">入力。ベクトル V の次元</param>
		/// <param name="v">入力。ベクトル V を格納している配列</param>
		/// <param name="format">入力。ベクトル V の要素を
		/// 印刷するために使用する書式文字列。
		/// 例. " {0, 9:F4}"</param>
		/// <param name="sw">入力。印字先のストリーム</param>
		public static void PrintVec(
			int n, double[] v, string format,
			System.IO.StreamWriter sw
		)
		{
			sw.Write("   ");
			for (int i = 0; i < n; i++)
				sw.Write(format, v[i]);

			sw.WriteLine("");
		}

		/// <summary>
		/// ベクトル V の内容をストリームに印字する。
		///要素を印字するとき、
		///標準のフォーマット " {0, 9:F4}" を使用します。
		/// </summary>
		/// <param name="n">入力。ベクトル V の次元</param>
		/// <param name="v">入力。ベクトル V を格納している配列</param>
		/// <param name="sw">入力。印字先のストリーム</param>
		public static void PrintVec(
			int n, double[] v, System.IO.StreamWriter sw
		)
		{
			PrintVec(n, v, " {0, 9:F4}", sw);
		}

		/// <summary>
		/// ベクトル V の内容をストリームに印字します。
		///成分の個数が大きいと折り返して印字します。
		/// </summary>
		/// <param name="n">入力。ベクトル V の次元</param>
		/// <param name="k">入力。n が大きいときに一行に何個の要素を
		///書くか</param>
		/// <param name="c">入力。各行の最初の k 個の成分を出力する
		///ときに出力する行頭文字、例えば ':'</param>
		/// <param name="v">入力。ベクトル V を格納している配列</param>
		/// <param name="format">入力。ベクトル V の要素を
		///を印刷するために使用する書式文字列。
		///例. " {0, 9:F4}"</param>
		/// <param name="sw">入力。印字先のストリーム</param>
		public static void PrintVec(
			int n, int k, char c, double[] v, string format,
			System.IO.StreamWriter sw
		)
		{
			sw.Write(" " + c + " ");
			int js = 0;
			while (js < n)
			{
				int je = Math.Min(n, js + k);
				for (int j = js; j < je; j++)
				{
					sw.Write(format, v[j]);
				}
				sw.WriteLine("");
				if (je >= n)
				{
					break;
				}
				else
				{
					js = je;
					sw.Write("   ");
				}
			}
		}

		/// <summary>
		/// ベクトル V の内容をストリームに印字します。
		///要素を印字するとき、
		///標準のフォーマット " {0, 9:F4}" を使用します。
		/// 成分の個数が大きいと折り返して印字します。
		/// </summary>
		/// <param name="n">入力。ベクトル V の次元</param>
		/// <param name="k">入力。n が大きいときに一行に何個の要素を
		///書くか</param>
		/// <param name="c">入力。各行の最初の k 個の成分を出力する
		///ときに出力する行頭文字、例えば ':'</param>
		/// <param name="v">入力。ベクトル V を格納している配列</param>
		/// <param name="sw">入力。印字先のストリーム</param>
		public static void PrintVec(
			int n, int k, char c, double[] v, System.IO.StreamWriter sw
		)
		{
			PrintVec(n, k, c, v, " {0, 9:F4}", sw);
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="str"></param>
		/// <param name="fmt"></param>
		/// <param name="al"></param>
		/// <param name="sb"></param>
		protected static void isBuild(string str, string fmt, ArrayList al, StringBuilder sb)
		{
			bool flag = false;
			al = new ArrayList();
			sb = new StringBuilder();

			for (int i = 0; i < fmt.Length; ++i)
			{
				if (flag)
				{
					if (fmt[i] == 's')
					{
						sb.Append(@"(\w+)");
						al.Add(typeof(string));
					}
					else if (fmt[i] == 'd')
					{
						sb.Append(@"(\d+)");
						al.Add(typeof(int));
					}

					else if (fmt[i] == 'f')
					{
						sb.Append(@"(\f+)");
						al.Add(typeof(double));
					}
					else
					{
						sb.Append('%');
						sb.Append(fmt[i]);
					}
					flag = false;
				}
				else
				{
					if (fmt[i] == '%')
						flag = true;
					else
						sb.Append(fmt[i]);
				}
			}
			sb.Append(".*");

		}


		/// <summary>
		/// 実験用につくったけど没
		/// </summary>
		/// <param name="str"></param>
		/// <param name="fmt"></param>
		/// <param name="d1"></param>
		/// <param name="d2"></param>
		/// <param name="d3"></param>
		/// <returns></returns>
		public static int sscanf(string str, string fmt, ref double d1, ref double d2, ref double d3)
		{
			ArrayList al = new ArrayList();
			StringBuilder sb = new StringBuilder();

			isBuild(str, fmt, al, sb);
			Match m = Regex.Match(str, sb.ToString());

			if (m.Success)
			{
				Type[] types = (Type[])al.ToArray(typeof(Type));
				d1 = double.Parse(m.Groups[1].Value);
				d2 = double.Parse(m.Groups[2].Value);
				d3 = double.Parse(m.Groups[3].Value);
				return types.Length;
			}
			return 0;
		}

	}

	/*
	/// <summary>
	/// 訪問者インターフェイス
	/// </summary>
	/// <typeparam name="KeyType">ノードキー</typeparam>
	/// <typeparam name="DataType">ノードデータ</typeparam>
	public abstract class TreeVisitorBase<KeyType, DataType>
	{
		abstract public void visit(TreeNode<KeyType, DataType> node);
	}

	/// <summary>
	/// 応対者インターフェイス
	/// </summary>
	/// <typeparam name="KeyType">ノードキー</typeparam>
	/// <typeparam name="DataType">ノードデータ</typeparam>
	public interface ITreeAccepter<KeyType, DataType>
	{
		void accept(TreeVisitorBase<KeyType, DataType> visitor);
	}

	/// <summary>
	/// 二分木
	/// </summary>
	/// <typeparam name="KeyType">ノードキー</typeparam>
	/// <typeparam name="DataType">ノードデータ</typeparam>
	public class TreeNode<KeyType, DataType> : ITreeAccepter<KeyType, DataType>
	{
		/// <summary>
		/// keyデータメンバ
		/// </summary>
		protected KeyType m_Key;
		/// <summary>
		/// Keyデータプロパティ
		/// </summary>
		public KeyType Key
		{
			get { return m_Key; }
			set { m_Key = value; }
		}
		/// <summary>
		/// ノードデータ
		/// </summary>
		protected DataType m_NodeData;
		/// <summary>
		/// Nodeデータプロパティ
		/// </summary>
		public DataType NodeData
		{
			set { m_NodeData = value; }
			get { return m_NodeData; }
		}
		// 
		protected TreeNode<KeyType, DataType> m_Parent;
		protected TreeNode<KeyType, DataType> m_Left;
		protected TreeNode<KeyType, DataType> m_Right;
		/// <summary>
		/// キー比較用に呼び出すデリゲート宣言
		/// </summary>
		public event System.Comparison<KeyType> Compare;

		/// <summary>
		/// コンストラクタ
		/// </summary>
		public TreeNode()
		{
			this.m_Parent = null;
			this.m_Left = null;
			this.m_Right = null;
		}

		public TreeNode(KeyType key, DataType data)
		{
			this.Key = key;
			this.NodeData = data;
			this.m_Left = null;
			this.m_Right = null;
		}

		/// <summary>
		/// ノード追加
		/// </summary>
		/// <param name="nodeData">追加するノードデータ</param>
		/// <returns>true成功,false失敗</returns>
		public bool AddNode(TreeNode<KeyType, DataType> nodeData)
		{
			// TODO:ノード追加処理 m_Parent
			this.m_Parent = InsertNode(this, nodeData);
			if (null == this.m_Parent)
			{
				return false;
			}
			return true;
		}

		protected TreeNode<KeyType, DataType> InsertNode(TreeNode<KeyType, DataType> targetNode, TreeNode<KeyType, DataType> nodeData)
		{

			if (targetNode == null)
			{
				targetNode = new TreeNode<KeyType, DataType>(nodeData.Key, nodeData.NodeData);
			}
			else
			{
				if (0 >= Compare(nodeData.Key, this.Key))
				{
					targetNode.m_Left = InsertNode(targetNode.m_Left, nodeData);
					targetNode.m_Left.m_Parent = this;
				}
				else
				{
					targetNode.m_Right = InsertNode(targetNode.m_Right, nodeData);
					targetNode.m_Right.m_Parent = this;
				}
			}
			return targetNode;
		}

		/// <summary>
		/// ノード削除
		/// </summary>
		/// <param name="targetKey">削除対象キー</param>
		/// <returns>true成功,false失敗</returns>
		public bool DeleteNode(KeyType targetKey)
		{
			// TODO:ノード削除処理
			return false;
		}

		public void accept(TreeVisitorBase<KeyType, DataType> Visitor)
		{
			Visitor.visit(this);
		}

		public TreeNode<KeyType, DataType> getLeftNode()
		{
			return this.m_Left;
		}

		public TreeNode<KeyType, DataType> getRightNode()
		{
			return this.m_Right;
		}

		public TreeNode<KeyType, DataType> getParetNode()
		{
			return this.m_Parent;
		}
	}
	*/
}
